# LpBugReporter - Is a python library for Launchdpad Bug Reporting.
#
# Copyright (C) 2010 zedtux <zedtux@zedroot.org>
#
# TuXtremSplit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# TuXtremSplit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TuXtremSplit; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

import logging

# The terminal has 8 colors with codes from 0 to 7
BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE = range(8)

# These are the sequences need to get colored ouput
RESET_SEQ = "\033[0m"
COLOR_SEQ = "\033[1;%dm"
BOLD_SEQ =  "\033[1m"

# The background is set with 40 plus the number of the color,
# and the foreground with 30
COLORS = {
    "WARNING":  COLOR_SEQ % (30 + YELLOW) + "WARNING" + RESET_SEQ,
    "INFO":     COLOR_SEQ % (30 + WHITE) + "INFO" + RESET_SEQ,
    "DEBUG":    COLOR_SEQ % (30 + BLUE) + "DEBUG" + RESET_SEQ,
    "CRITICAL": COLOR_SEQ % (30 + YELLOW) + "CRITICAL" + RESET_SEQ,
    "ERROR":    COLOR_SEQ % (30 + RED) + "ERROR" + RESET_SEQ,
}

class ColoredFormatter(logging.Formatter):
    def __init__(self, msg, use_color=True):
        logging.Formatter.__init__(self, msg)
        self.use_color = use_color

    def format(self, record):
        if self.use_color:
            record.levelname = COLORS.get(record.levelname, record.levelname)
        return logging.Formatter.format(self, record)

# Custom logger class with multiple destinations
class AppLogger(logging.Logger):
    COLOR_FORMAT = "["+BOLD_SEQ+"%(name)s"+RESET_SEQ+"][%(levelname)s] %(message)s ("+BOLD_SEQ+"%(filename)s"+RESET_SEQ+":%(lineno)d)"
    NO_COLOR_FORMAT = "[%(name)s][%(levelname)s] %(message)s (%(filename)s:%(lineno)d)"
    LOG_FILE_HANDLER = None

    def __init__(self, name):
        logging.Logger.__init__(self, name)
        # Add two handlers, a stderr one, and a file one
        color_formatter = ColoredFormatter(AppLogger.COLOR_FORMAT)
        no_color_formatter = ColoredFormatter(AppLogger.NO_COLOR_FORMAT, False)
        console = logging.StreamHandler()
        console.setFormatter(color_formatter)
        self.addHandler(console)
        return

def enable_debugging():
    logging.getLogger().setLevel(logging.DEBUG)

logging.setLoggerClass(AppLogger)
