#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# LpBugReporter - Is a python library for Launchdpad Bug Reporting.
#
# Copyright (C) 2010 zedtux <zedtux@zedroot.org>
#
# TuXtremSplit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# TuXtremSplit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with TuXtremSplit; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA

import time
import logging
import threading

from launchpadlib.launchpad import Launchpad
from lazr.restfulclient.resource import Collection

log = logging.getLogger("LpBugReporter")

STATUSES = ["New", "Incomplete", "Invalid", "Won't Fix", "Confirmed", "Triaged", "In Progress", "Fix Committed"]

class LpBrThread(threading.Thread):
    
    """
    Thread that do stuffs.
    """
    def __init__(self, window):
        """
        Constructor
        """
        super(LpBrThread, self).__init__()
        self._stopevent = threading.Event()
        
        self.__window = window
        self.__lp_connector = None
        self.__project = None
        self.__tasks = None
        self.__cache_dir = ""
        self.__project_name = ""
        self.__is_running = False
    
    # property: cache_dir
    def get_cache_dir(self):
        return self.__cache_dir
    def set_cache_dir(self, cache_dir):
        self.__cache_dir = cache_dir
    cache_dir = property(get_cache_dir, set_cache_dir)
    
    # property: project_name
    def get_project_name(self):
        return self.__project_name
    def set_project_name(self, project_name):
        self.__project_name = project_name
    project_name = property(get_project_name, set_project_name)
    
    # property is_running
    def get_is_running(self):
        return self.__is_running
    def set_is_running(self, running):
        self.__is_running = running
    is_running = property(get_is_running, set_is_running)
    
    def stop(self):
        """
        Stop running thread
        """
        self.is_running = False
        self._stopevent.set()
    
    def run(self):
        """
        Execution code of the thread.
        """
        self.is_running = True
        
        self.__lp_connector = Launchpad.login_anonymously("LpBugReporter",
                                                          "production",
                                                          self.cache_dir)
        if self.__lp_connector is None:
            log.error("Unable to connect to Launchdpad.")
            self.__window.on_error("Unable to connect to Launchpad.")
            return
        else:
            log.debug("Connected to Launchdpad.")
        
        self.__project = self.__lp_connector.projects[self.project]
        if self.__project is None:
            log.error("Unable to find the project %s." % self.project)
            self.__window.on_error("Unable to find the project on Launchpad.")
            return
        else:
            log.debug("Project %s found." % self.project)
        
        try:
            self.__tasks = self.__project.searchTasks(status=STATUSES)
        except:
            pass
        
        if self.__tasks is None:
            self.__window.on_error("Unable to fetch bugs list.")
            return
        else:
            self.__window.row_max = int(self.__tasks._wadl_resource.representation['total_size'])
            self.__window.switch_view()
            
            if self.__window.row_max > 0:
                
                for task in self.__tasks:
                    
                    if not self.is_running:
                        self.__window.enable_start_stop_fetching_button()
                        break
                    
                    self.__window.append_row([task.bug.id,
                                              task.bug.owner.name,
                                              task.bug.title[0:60] if len(task.bug.title) > 30 else task.bug.title,
                                              "Double-click to go to the bug %s" % task.bug.id])
                    time.sleep(0.1) # Refresh gui
            else:
                self.__window.switch_view()
